local green = '|cff20ff20%s|r'
local yellow = '|cffffff20%s|r'
local red = '|cffff2020%s|r'
local normal = '|cffffd200%s|r'
local monthOffset = 0

local TEXTURE_STRING = '|T%s:80|t'

-- Event Types
local CALENDAR_EVENTTYPE_RAID		= 1;
local CALENDAR_EVENTTYPE_DUNGEON	= 2;
local CALENDAR_EVENTTYPE_PVP		= 3;
local CALENDAR_EVENTTYPE_MEETING	= 4;
local CALENDAR_EVENTTYPE_OTHER		= 5;

-- Invite Statuses
local CALENDAR_INVITESTATUS_INVITED		= 1;
local CALENDAR_INVITESTATUS_ACCEPTED	= 2;
local CALENDAR_INVITESTATUS_DECLINED	= 3;
local CALENDAR_INVITESTATUS_CONFIRMED	= 4;
local CALENDAR_INVITESTATUS_OUT			= 5;
local CALENDAR_INVITESTATUS_STANDBY		= 6;

local CALENDAR_INVITESTATUS_NAMES = {
	[CALENDAR_INVITESTATUS_CONFIRMED]	= string.format(green, CALENDAR_STATUS_CONFIRMED),
	[CALENDAR_INVITESTATUS_ACCEPTED]	= string.format(green, CALENDAR_STATUS_ACCEPTED),
	[CALENDAR_INVITESTATUS_DECLINED]	= string.format(red, CALENDAR_STATUS_DECLINED),
	[CALENDAR_INVITESTATUS_OUT]			= string.format(red, CALENDAR_STATUS_OUT),
	[CALENDAR_INVITESTATUS_STANDBY]		= string.format(yellow, CALENDAR_STATUS_STANDBY),
	[CALENDAR_INVITESTATUS_INVITED]		= string.format(normal, CALENDAR_STATUS_INVITED),
};

local CALENDAR_HOLIDAY_PATH_FORMAT = {
	["START"]			= "Interface\\Calendar\\Holidays\\%sStart",
	["ONGOING"]			= "Interface\\Calendar\\Holidays\\%sStart",
	["END"]				= "Interface\\Calendar\\Holidays\\%sEnd",
	["INFO"]			= "Interface\\Calendar\\Holidays\\%sInfo",
	[""]				= "Interface\\Calendar\\Holidays\\%s",
};

local CALENDAR_EVENTTYPE_TEXTURE_PATHS = {
	[CALENDAR_EVENTTYPE_RAID]		= "Interface\\LFGFrame\\LFGIcon-",
	[CALENDAR_EVENTTYPE_DUNGEON]	= "Interface\\LFGFrame\\LFGIcon-",
};
local CALENDAR_EVENTTYPE_TEXTURES = {
	[CALENDAR_EVENTTYPE_RAID]		= "Interface\\LFGFrame\\LFGIcon-Raid",
	[CALENDAR_EVENTTYPE_DUNGEON]	= "Interface\\LFGFrame\\LFGIcon-Dungeon",
	[CALENDAR_EVENTTYPE_PVP]		= "Interface\\Calendar\\UI-Calendar-Event-PVP",
	[CALENDAR_EVENTTYPE_MEETING]	= "Interface\\Calendar\\MeetingIcon",
	[CALENDAR_EVENTTYPE_OTHER]		= "Interface\\Calendar\\UI-Calendar-Event-Other",
};
local CALENDAR_CALENDARTYPE_TEXTURES = {
	["HOLIDAY"] = {
		["START"]			= "Interface\\Calendar\\Holidays\\Calendar_DefaultHoliday",
		["END"]				= "Interface\\Calendar\\Holidays\\Calendar_DefaultHoliday",
		["INFO"]			= "Interface\\Calendar\\Holidays\\Calendar_DefaultHoliday",
	},
};

local Calendar = CreateFrame("Frame")
function Calendar:Print(...) ChatFrame1:AddMessage(string.join(" ", "|cFF33FF99Calendar|r:", ...)) end

function Calendar:CreateBroker()
	self.obj = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_CalendarEvents", {
		icon = "Interface\\AddOns\\Broker_CalendarEvents\\icon",
		label = "Events",
		type = "data source",
		text  = "0",
		
		OnClick = function(self, btn)
			GameTimeFrame_OnClick(GameTimeFrame)
		end,
		
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine or (CalendarFrame and CalendarFrame:IsShown()) then return end
			tooltip:AddLine("Todays Events:")
			tooltip:AddLine('')
			local _, _, day = CalendarGetDate()
			for i=1, CalendarGetNumDayEvents(monthOffset, day) do
				Calendar:addevent(tooltip,i)
			end
			
			if ( GameTimeCalendarInvitesTexture:IsShown() ) then
				tooltip:AddLine(GAMETIME_TOOLTIP_CALENDAR_INVITES);
				if ( CalendarFrame and not CalendarFrame:IsShown() ) then
					tooltip:AddLine(" ");
					tooltip:AddLine(GAMETIME_TOOLTIP_TOGGLE_CALENDAR);
				end
			else
				if ( not TimeManagerClockButton or not TimeManagerClockButton:IsVisible() or TimeManager_IsAlarmFiring() ) then
					tooltip:AddLine(GameTime_GetGameTime(true), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
					tooltip:AddLine(" ");
				end
				tooltip:AddLine(GAMETIME_TOOLTIP_TOGGLE_CALENDAR);
			end
			Calendar:UpdateText()
		end,
	})
end
local counter, delay = 0, 60
function Calendar:OnUpdate(elapsed)
	counter = counter + elapsed
	if counter >= delay then
		Calendar:UpdateText()
		counter = 0
	end
end

function Calendar:addevent(tooltip,i)
	local TimeOfEvent
	local _, _, day = CalendarGetDate()
	local title, hour, minute, calendarType, sequenceType, eventType, texture, modStatus, inviteStatus, invitedBy, difficulty = CalendarGetDayEvent(monthOffset, day, i)
	if ( calendarType == "PLAYER" or calendarType == "GUILD" or calendarType == "ARENA" ) then
		if sequenceType == '' or sequenceType == 'START' then
			local currentHour, currentMinute = GetGameTime()
			local minuteleft = ((hour * 60) + minute) - ((currentHour * 60) + currentMinute)
			if minuteleft < 0 then
				TimeOfEvent = string.format(red, string.format('%02d:%02d', hour, minute))
			elseif minuteleft < 60 then
				TimeOfEvent = string.format(yellow, string.format('%02d:%02d', hour, minute))
			else
				TimeOfEvent = string.format(green, string.format('%02d:%02d', hour, minute))
			end
		elseif sequenceType == 'ONGOING' then
			TimeOfEvent = RUNE_COST_ONGOING
		else --end
			TimeOfEvent = string.format(CALENDAR_EVENTNAME_FORMAT_END, title)
		end
		local texture, texturestring = self:GetTextureFile(texture, calendarType, sequenceType, eventType)
		tooltip:AddDoubleLine(texturestring..title,TimeOfEvent)
		if ( calendarType == "PLAYER" or calendarType == "GUILD" or calendarType == "ARENA" ) then -- needed if i add holidays into the tooltip
			local isGuildWide = CalendarContextEventIsGuildWide(monthOffset, day, i);
			if ( UnitIsUnit("player", invitedBy) ) then
				if ( isGuildWide ) then
					tooltip:AddLine(CALENDAR_ANNOUNCEMENT_CREATEDBY_YOURSELF,NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
				else
					tooltip:AddLine(CALENDAR_INVITEDBY_YOURSELF, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
				end
			elseif ( invitedBy ~= "" ) then
				if ( isGuildWide ) then
					tooltip:AddDoubleLine(string.format(CALENDAR_ANNOUNCEMENT_CREATEDBY_PLAYER, invitedBy), CALENDAR_INVITESTATUS_NAMES[inviteStatus])
				else
					tooltip:AddDoubleLine(string.format(CALENDAR_INVITEDBY_PLAYER, invitedBy), CALENDAR_INVITESTATUS_NAMES[inviteStatus])
				end
			else
				tooltip:AddLine(CALENDAR_INVITESTATUS_NAMES[inviteStatus])
			end
		end
		tooltip:AddLine(' ')
	end
end

function Calendar:GetTextureFile(textureName, calendarType, sequenceType, eventType)
	local texture
	if ( textureName and textureName ~= "" ) then
		if calendarType == "HOLIDAY" then
			texture = string.format(CALENDAR_HOLIDAY_PATH_FORMAT[sequenceType], textureName)
		elseif CALENDAR_EVENTTYPE_TEXTURE_PATHS[eventType] then -- Raids/Dungeon
			texture = CALENDAR_EVENTTYPE_TEXTURE_PATHS[eventType]..textureName
		elseif CALENDAR_EVENTTYPE_TEXTURES[eventType] then
			texture = CALENDAR_EVENTTYPE_TEXTURES[eventType]
		end
	elseif CALENDAR_CALENDARTYPE_TEXTURES[calendarType] and CALENDAR_CALENDARTYPE_TEXTURES[calendarType][sequenceType] then
	
	elseif CALENDAR_EVENTTYPE_TEXTURES[eventType] then
		texture = CALENDAR_EVENTTYPE_TEXTURES[eventType]
	end
	return texture, string.format(TEXTURE_STRING, texture)
end

function Calendar:ADDON_LOADED(event, addon)
	if addon == "Broker_CalendarEvents" then
		--Calendar_LoadUI()
		self:UnregisterEvent("ADDON_LOADED")
		self:CreateBroker()
		--self:UpdateEventCache()
		self:UpdateText()
		self.ADDON_LOADED = nil
		GameTimeFrame:Hide()
		GameTimeFrame.Show = function() end
	end
end

--[[
function Calendar:UpdateEventCache()
	self.day = select(3,CalendarGetDate())
	self.events = CalendarGetNumDayEvents(monthOffset, self.day)
	if self.events > 0 then
		self.eventscan = 1
		--CalendarOpenEvent(monthOffset, self.day, self.eventscan)
	end
end

function Calendar:CALENDAR_OPEN_EVENT(event, ...)
	local title, description, creator, eventType, arg1, arg2, TextureIDforEventType, arg3, MonthStart, dayStart, YearStart, HourStart, MinStart, arg5, arg6, arg7, arg8, arg9, arg10, isLocked, arg11, arg12, arg13 = CalendarGetEventInfo(monthOffset, self.day, self.eventscan)
	self:Print(title)
	if self.eventscan < self.events then
		self.eventscan = self.eventscan + 1
		--CalendarOpenEvent(monthOffset, self.day, self.eventscan)
	end
end
]]--

function Calendar:CALENDAR_NEW_EVENT(event, ...)
	self:UpdateText()
end

function Calendar:CALENDAR_UPDATE_EVENT(event, ...)
	self:UpdateText()
end

function Calendar:CALENDAR_UPDATE_EVENT_LIST(event, ...)
	self:UpdateText()
end

function Calendar:UpdateText()
	local _, _, day = CalendarGetDate()
	local events = CalendarGetNumDayEvents(0, day)
	for i=1, events do
		if 'HOLIDAY' == select(4, CalendarGetDayEvent(monthOffset, day, i)) then
			events = events - 1
		end
	end
	self.obj.text = string.format(normal, events)
end

Calendar:SetScript("OnUpdate", Calendar.OnUpdate)
Calendar:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
Calendar:RegisterEvent("ADDON_LOADED")
Calendar:RegisterEvent("CALENDAR_NEW_EVENT")
		--Calendar:RegisterEvent("CALENDAR_OPEN_EVENT")
Calendar:RegisterEvent("CALENDAR_UPDATE_EVENT")
Calendar:RegisterEvent("CALENDAR_UPDATE_EVENT_LIST")
